//////////////////////////////////////////////////////////////////////////////////////////////////
// 
// File: alphaTexQuad.cpp
// 
// Author: Kyoung Shin Park
//
// Desc: Renders a textured quad & an alpha blended texture quad.  
//          
//////////////////////////////////////////////////////////////////////////////////////////////////

#include "d3dUtility.h"
#include "Quad.h"

// Globals
IDirect3DDevice9*     Device = 0; 

const int Width  = 640;
const int Height = 480;

Quad* MainBox = 0;
Quad* PopupBox = 0;
IDirect3DTexture9*      mainTex  = 0;
IDirect3DTexture9*      popupTex  = 0;
BOOL popupToggle = false;

// Framework Functions
bool Setup()
{

	MainBox = new Quad(Device);
	D3DXCreateTextureFromFile( Device, "main.jpg", &mainTex );

	PopupBox = new Quad(Device);
	D3DXCreateTextureFromFile( Device, "nice.dds", &popupTex );

	// set texture filters
	Device->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	Device->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	Device->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_POINT);

	// set alpha blending
	Device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	Device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	// don't use lighting for this sample.
	Device->SetRenderState(D3DRS_LIGHTING, false);

	// set the projection matrix.
	D3DXMATRIX proj;
	D3DXMatrixPerspectiveFovLH(
			&proj,
			D3DX_PI * 0.5f, // 90 - degree
			(float)Width / (float)Height,
			1.0f,
			1000.0f);
	Device->SetTransform(D3DTS_PROJECTION, &proj);

	return true;
}

void Cleanup()
{
	d3d::Delete<Quad*>(MainBox);
	d3d::Delete<Quad*>(PopupBox);
	d3d::Release<IDirect3DTexture9*>(mainTex);
	d3d::Release<IDirect3DTexture9*>(popupTex);
}

bool Display(float timeDelta)
{
	if ( Device ) {
		Device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0);
		Device->BeginScene();

		D3DXMATRIX I;
		D3DXMatrixIdentity(&I);
		MainBox->draw(&I, 0, mainTex);

		if ( popupToggle ) {
			Device->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
			D3DXMATRIX S;
			D3DXMatrixScaling(&S, 0.5, 0.7, 1.0);
			PopupBox->draw(&S, 0, popupTex);
			Device->SetRenderState( D3DRS_ALPHABLENDENABLE, false );
		}

		Device->EndScene();
		Device->Present(0, 0, 0, 0);
	}
	return true;
}


//
// WndProc
//
LRESULT CALLBACK d3d::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_DESTROY:
		::PostQuitMessage(0);
		break;
		
	case WM_KEYDOWN:
		if( wParam == VK_ESCAPE )
			::DestroyWindow(hwnd);
		else if( wParam == VK_SPACE )
			popupToggle = ~popupToggle;
		break;
	}
	return ::DefWindowProc(hwnd, msg, wParam, lParam);
}

//
// WinMain
//
int WINAPI WinMain(HINSTANCE hinstance,
				   HINSTANCE prevInstance, 
				   PSTR cmdLine,
				   int showCmd)
{
	if(!d3d::InitD3D(hinstance,
		Width, Height, true, D3DDEVTYPE_HAL, &Device))
	{
		::MessageBox(0, "InitD3D() - FAILED", 0, 0);
		return 0;
	}
		
	if(!Setup())
	{
		::MessageBox(0, "Setup() - FAILED", 0, 0);
		return 0;
	}

	d3d::EnterMsgLoop( Display );

	Cleanup();

	Device->Release();

	return 0;
}